# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"image_box": "box.ipynb",
         "get_array": "box.ipynb",
         "cat_box": "box.ipynb",
         "mcat_box": "box.ipynb",
         "bbox_box": "box.ipynb",
         "DataBox": "core.ipynb",
         "create_datablock": "core.ipynb",
         "create_dataloaders": "core.ipynb",
         "dataloaders_seed": "core.ipynb",
         "function": "utils.ipynb",
         "T": "utils.ipynb",
         "identity": "utils.ipynb",
         "simplify": "utils.ipynb",
         "listify": "utils.ipynb",
         "setify": "utils.ipynb",
         "tuplify": "utils.ipynb",
         "merge_tfms": "utils.ipynb",
         "compose": "utils.ipynb",
         "pipe": "utils.ipynb",
         "flow": "utils.ipynb",
         "get_files": "utils.ipynb",
         "Path.find": "utils.ipynb",
         "Path.decompress": "utils.ipynb",
         "Path.compress": "utils.ipynb",
         "Path.download": "utils.ipynb",
         "save_array": "utils.ipynb"}

modules = ["box.py",
           "core.py",
           "utils.py"]

doc_url = "//"

git_url = "https://github.com/limiteinductive/Databox/tree/main/"

def custom_doc_links(name): return None



# AUTOGENERATED! DO NOT EDIT! File to edit: box.ipynb (unless otherwise specified).

__all__ = ['image_box', 'get_array', 'cat_box', 'mcat_box', 'bbox_box']

# Cell
from fastai.vision.all import *

from .core import *
from .utils import *

# Cell
def image_box(size = 224, folder = '', aug_kwargs = None, **kwargs):
    """Make a DataBox from an ImageBlock."""
    if aug_kwargs is not None:
        kwargs = merge_tfms(
            {'batch_tfms': [*aug_transforms(size=size, **aug_kwargs)]}, kwargs
        )

    return DataBox(
        ImageBlock,
        lambda x: Path(x.parent) / folder / x.name,
        **merge_tfms(kwargs, {'item_tfms': Resize(size)})
    )

# Cell
def get_array(suffix):
    """Return a getter that loads a numpy array with given suffix."""
    return lambda x: np.load(x.with_suffix(f'{suffix}.npy'), allow_pickle=True)

# Cell
def cat_box(suffix=''):
    """Make a Category Box from the categorial array of the dataset."""
    return DataBox(CategoryBlock, pipe(get_array(suffix), str))

# Cell
from ast import literal_eval

def mcat_box(suffix=''):
    """Make a Category Box from the categorial array of the dataset."""
    return DataBox(MultiCategoryBlock, lambda x: literal_eval(get_array(suffix)(x).tolist()))

# Cell
def bbox_box():
    return DataBox(
    (BBoxBlock, BBoxLblBlock),
    [get_array('.box'), get_array('.label')]
)


# AUTOGENERATED! DO NOT EDIT! File to edit: utils.ipynb (unless otherwise specified).

__all__ = ['function', 'T', 'identity', 'simplify', 'listify', 'setify', 'tuplify', 'merge_tfms', 'compose', 'pipe',
           'flow', 'get_files', 'save_array']

# Cell
import os
import numpy as np
from fastcore.all import *
import zipfile

from typing import Iterable, TypeVar, Generator
from plum import dispatch
from pathlib import Path
from functools import reduce

function = type(lambda: ())
T = TypeVar('T')

# Cell
def identity(x: T) -> T:
    """Indentity function."""

    return x

# Cell
def simplify(x):
    """Return an object of an iterable if it is lonely."""


    @dispatch
    def _simplify(x):
        if callable(x):
            try:
                return x()
            except TypeError:
                pass
        return x

    @dispatch
    def _simplify(i: Iterable): return next(i.__iter__()) if len(i) == 1 else i

    return _simplify(x)

# Cell
def listify(x, *args):
    """Convert `x` to a `list`."""
    if args:
        x = (x,) + args

    if x is None:
        result = []
    elif isinstance(x, list): result = x
    elif isinstance(x, str) or hasattr(x, "__array__") or hasattr(x, "iloc"):
        result = [x]
    elif isinstance(x, (Iterable, Generator)):
        result = list(x)
    else:
        result = [x]

    return result

# Cell
def setify(x, *args):
    """Convert `x` to a `set`."""

    return set(listify(x, *args))

# Cell
def tuplify(x, *args):
    """Convert `x` to a `tuple`."""

    return tuple(listify(x, *args))

# Cell
def merge_tfms(*tfms):
    """Merge two dictionnaries by stacking common key into list."""

    def _merge_tfms(tf1, tf2):
        return {
            k: simplify(listify(setify(listify(tf1.get(k)) + listify(tf2.get(k)))))
            for k in {**tf1, **tf2}
        }

    return reduce(_merge_tfms, tfms, dict())

# Cell
def compose(*functions):
    """Compose an arbitrary number of functions."""

    def _compose(fn1, fn2):
        return lambda x: fn1(fn2(x))

    return reduce(_compose, functions, identity)

# Cell
def pipe(*functions):
    """Pipe an arbitrary number of functions."""

    return compose(*functions[::-1])

# Cell
def flow(data, *functions):
    """Flow `data` through a list of functions."""

    return pipe(*functions)(data)

# Cell
def get_files(path, extensions=None, recurse=False, folders=None, followlinks=True):
    """Get all those file names."""
    path = Path(path)
    folders = listify(folders)
    extensions = setify(extensions)
    extensions = {e.lower() for e in extensions}

    def simple_getter(p, fs, extensions=None):
        p = Path(p)
        res = [
            p / f
            for f in fs
            if not f.startswith(".")
            and ((not extensions) or f'.{f.split(".")[-1].lower()}' in extensions)
        ]
        return res

    if recurse:
        result = []
        for i, (p, d, f) in enumerate(os.walk(path, followlinks=followlinks)):
            if len(folders) != 0 and i == 0:
                d[:] = [o for o in d if o in folders]
            else:
                d[:] = [o for o in d if not o.startswith(".")]
            if len(folders) != 0 and i == 0 and "." not in folders:
                continue
            result += simple_getter(p, f, extensions)
    else:
        f = [o.name for o in os.scandir(path) if o.is_file()]
        result = simple_getter(path, f, extensions)
    return list(map(str, result))

# Cell
from fuzzywuzzy.process import extractOne

@patch
def find(self: Path, match: str):
    """Find the matching element in path according to Levenshtein Distance."""
    return Path(extractOne(match, map(str, self.ls()))[0])

# Cell
@patch
def decompress(self: Path, dest='.'):

    if self.suffix == '.zip':
        with zipfile.ZipFile(self, 'r') as f:
            f.extractall(dest)
    elif self.suffix == 'tar':
        print('tar')
    else:
        raise ValueError('extension of the file is not supported')

# Cell
@patch
def compress(self: Path, dest='.', keep_copy=True):

    return 'I compress.'

# Cell
from fastdownload import FastDownload

@patch
def download(self: Path, url, dest='tmp/'):
    """TODO ça marche pas."""

    FastDownload(base=dest).get(url)

# Cell
def save_array(array, fname, suffix):
    """Save an array with the given name and suffix."""
    if not suffix.startswith("."):
        suffix = "." + suffix

    fname = Path(fname)

    return np.save(array, fname.with_suffix(suffix))


from .core import *
from .utils import *
from .box import *

# exterior imports
import numpy as np
import pandas as pd

from pathlib import Path
from pandas import read_csv


from fastai.vision.all import *
from .core import set_seed

path = Path('.')


# AUTOGENERATED! DO NOT EDIT! File to edit: core.ipynb (unless otherwise specified).

__all__ = ['DataBox', 'create_datablock', 'create_dataloaders', 'dataloaders_seed']

# Cell
from operator import add
from functools import reduce

from fastai.vision.all import *

from .utils import *

# Cell
class DataBox:

    def __init__(self, block=(), getter=None, **tfms):

        self.block = tuplify(block)
        self.getter = getter
        self.tfms = tfms

        if not tfms: self.tfms = {}


    def __repr__(self):

        return f"""block: {self.block}, getter: {self.getter}, tfms: {self.tfms}"""


    def __len__(self) -> int:
        return len(tuplify(self.getter))

    def __add__(self, other):
        blocks = self.block + other.block
        getters = listify(self.getter) + listify(other.getter)
        tfms = merge_tfms(self.tfms, other.tfms)

        return DataBox(blocks, getters, **tfms)


# Cell

#export
def create_datablock(input_box, output_box, **kwargs):

    ibox = reduce(add, map(simplify, listify(input_box)), DataBox())
    obox = reduce(add, map(simplify, listify(output_box)), DataBox())

    both_box = ibox + obox

    return DataBlock(
        n_inp=len(ibox),
        blocks=both_box.block,
        get_x=ibox.getter,
        get_y=obox.getter,
        get_items=partial(get_image_files, recurse=False),
        **merge_tfms(both_box.tfms, kwargs)
    )


# Cell
def create_dataloaders(input_box, output_box, path='.', bs=None, seed=None, **kwargs):

    dls = DataBlock(input_box, output_box, **kwargs).dataloaders(path, bs)

    if seed:
        dataloaders_seed(dls, seed)

    return dls


# Cell
def dataloaders_seed(dls, seed=0):
    """Set a random seed for all aspects of a Fastai Learner."""
    random.seed(seed)
    dls.rng.seed(seed)
    np.random.seed(seed)
    torch.manual_seed(seed)
    torch.backends.cudnn.deterministic = True
    torch.backends.cudnn.benchmark = False
    if torch.cuda.is_available(): torch.cuda.manual_seed_all(seed)